@@ -4,13 +4,12 @@
 ;; This file consists of excepts from Oleg Kiselyov's myenv.scm and
 ;; SSAX.scm.   They are macros and procedures only required for testing.
 ;;
-;; $Id: test-common.scm,v 1.2 2006-10-07 23:37:26 shirok Exp $
 
 ;; Excerpt from myenv.scm --------------------------------------
 
 (define-macro (assure exp error-msg) `(assert ,exp report: ,error-msg))
 
-    
+
 (define (cout . args)
   (for-each (lambda (x)
               (if (procedure? x) (x) (display x)))
@@ -53,7 +52,9 @@
 
 ;; Excerpt from SSAX.scm --------------------------------------
 
-(define-macro run-test
+;; Kludge - trans.scm removes run-test definition.  We use different
+;; name to prevent this from removed.
+(define-macro %run-test
   (lambda body
     (define (re-write body)
       (cond
