@@ -1,12 +1,12 @@
 /*
  * string.c - string implementation
  *
- *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -76,7 +76,6 @@ static ScmStringBody *make_str_body(int len, int siz, const char *p, int flags)
 #define DUMP_LENGTH   50
 
 /* for debug */
-#if SCM_DEBUG_HELPER
 void Scm_StringDump(FILE *out, ScmObj str)
 {
     int i;
@@ -95,9 +94,8 @@ void Scm_StringDump(FILE *out, ScmObj str)
         fputs("...\"\n", out);
     } else {
         fputs("\"\n", out);
-    }       
+    }
 }
-#endif /*SCM_DEBUG_HELPER*/
 
 /*
  * Multibyte length calculation
@@ -172,7 +170,7 @@ ScmObj Scm_MakeString(const char *str, int size, int len, int flags)
     } else {
         if (len < 0) len = count_length(str, size);
     }
-    
+
     if (flags & SCM_STRING_COPYING) {
         char *nstr = SCM_NEW_ATOMIC2(char *, size + 1);
         memcpy(nstr, str, size);
@@ -207,7 +205,7 @@ ScmObj Scm_ListToString(ScmObj chars)
     char *buf, *bufp;
 
     SCM_FOR_EACH(cp, chars) {
-        if (!SCM_CHARP(SCM_CAR(cp))) 
+        if (!SCM_CHARP(SCM_CAR(cp)))
             Scm_Error("character required, but got %S", SCM_CAR(cp));
         ch = SCM_CHAR_VALUE(SCM_CAR(cp));
         size += SCM_CHAR_NBYTES(ch);
@@ -284,7 +282,7 @@ const char *Scm_GetStringContent(ScmString *str,
 
 /* Copy string.  You can modify the flags of the newly created string
    by FLAGS and MASK arguments; for the bits set in MASK, corresponding
-   bits in FLAGS are copied to the new string, and for other bits, the  
+   bits in FLAGS are copied to the new string, and for other bits, the
    original flags are copied.
 
    The typical semantics of copy-string is achieved by passing 0 to
@@ -303,7 +301,7 @@ ScmObj Scm_CopyStringWithFlags(ScmString *x, int flags, int mask)
     const char *start = SCM_STRING_BODY_START(b);
     int newflags = ((SCM_STRING_BODY_FLAGS(b) & ~mask)
                     | (flags & mask));
-        
+
     return SCM_OBJ(make_str(len, size, start, newflags));
 }
 
@@ -355,7 +353,7 @@ ScmObj Scm_StringIncompleteToComplete(ScmString *x,
                 } else {
                     SCM_CHAR_GET(p, ch);
                 }
-                
+
                 if (ch != SCM_CHAR_INVALID) {
                     Scm_DStringPutc(&ds, ch);
                     p += SCM_CHAR_NBYTES(ch);
@@ -369,7 +367,7 @@ ScmObj Scm_StringIncompleteToComplete(ScmString *x,
             r = Scm_DStringGet(&ds, 0);
         }
     }
-    
+
     return r;
 }
 
@@ -458,7 +456,7 @@ int Scm_StringCiCmp(ScmString *x, ScmString *y)
     const char *px, *py;
     const ScmStringBody *xb = SCM_STRING_BODY(x);
     const ScmStringBody *yb = SCM_STRING_BODY(y);
-    
+
     if ((SCM_STRING_BODY_FLAGS(xb)^SCM_STRING_BODY_FLAGS(yb))&SCM_STRING_INCOMPLETE) {
         Scm_Error("cannot compare incomplete strings in case-insensitive way: %S, %S",
                   SCM_OBJ(x), SCM_OBJ(y));
@@ -467,7 +465,7 @@ int Scm_StringCiCmp(ScmString *x, ScmString *y)
     sizy = SCM_STRING_BODY_SIZE(yb); leny = SCM_STRING_BODY_LENGTH(yb);
     px = SCM_STRING_BODY_START(xb);
     py = SCM_STRING_BODY_START(yb);
-    
+
     if (sizx == lenx && sizy == leny) {
         return sb_strcasecmp(px, sizx, py, sizy);
     } else {
@@ -483,7 +481,7 @@ int Scm_StringCiCmp(ScmString *x, ScmString *y)
 static const char *forward_pos(const char *current, int offset)
 {
     int n;
-    
+
     while (offset--) {
         n = SCM_CHAR_NFOLLOWS(*current);
         current += n + 1;
@@ -549,9 +547,8 @@ int Scm_StringByteRef(ScmString *str, int offset, int range_error)
    case, the return value points the location past the string body,
    but it is necessary sometimes to do a pointer arithmetic with the
    returned values. */
-const char *Scm_StringPosition(ScmString *str, int offset)
+const char *Scm_StringBodyPosition(const ScmStringBody *b, int offset)
 {
-    const ScmStringBody *b = SCM_STRING_BODY(str);
     if (offset < 0 || offset > (int)SCM_STRING_BODY_LENGTH(b)) {
         Scm_Error("argument out of range: %d", offset);
     }
@@ -562,6 +559,14 @@ const char *Scm_StringPosition(ScmString *str, int offset)
     }
 }
 
+/* This is old API and now DEPRECATED.  It's difficult to use this safely,
+   since you don't have a way to get the string length consistent at the
+   moment you call this function.   Use Scm_StringBodyPosition instead. */
+const char *Scm_StringPosition(ScmString *str, int offset)
+{
+    return Scm_StringBodyPosition(SCM_STRING_BODY(str), offset);
+}
+
 /*----------------------------------------------------------------
  * Concatenation
  */
@@ -595,7 +600,7 @@ ScmObj Scm_StringAppendC(ScmString *x, const char *str, int sizey, int leny)
 
     if (sizey < 0) count_size_and_length(str, &sizey, &leny);
     else if (leny < 0) leny = count_length(str, sizey);
-    
+
     p = SCM_NEW_ATOMIC2(char *, sizex + sizey + 1);
     memcpy(p, xb->start, sizex);
     memcpy(p+sizex, str, sizey);
@@ -807,7 +812,7 @@ ScmObj Scm_MaybeSubstring(ScmString *x, ScmObj start, ScmObj end)
 {
     int istart, iend;
     const ScmStringBody *xb = SCM_STRING_BODY(x);
-    if (SCM_UNBOUNDP(start) || SCM_UNDEFINEDP(start)) {
+    if (SCM_UNBOUNDP(start) || SCM_UNDEFINEDP(start) || SCM_FALSEP(start)) {
         istart = 0;
     } else {
         if (!SCM_INTP(start))
@@ -815,7 +820,7 @@ ScmObj Scm_MaybeSubstring(ScmString *x, ScmObj start, ScmObj end)
         istart = SCM_INT_VALUE(start);
     }
 
-    if (SCM_UNBOUNDP(end) || SCM_UNDEFINEDP(end)) {
+    if (SCM_UNBOUNDP(end) || SCM_UNDEFINEDP(end) || SCM_FALSEP(end)) {
         if (istart == 0) return SCM_OBJ(x);
         iend = SCM_STRING_BODY_LENGTH(xb);
     } else {
@@ -844,7 +849,7 @@ ScmObj Scm_StringSplitByChar(ScmString *str, ScmChar ch)
         /* TODO: fix the policy of handling incomplete string */
         Scm_Error("incomplete string not accepted: %S", str);
     }
-    
+
     while (p < e) {
         ScmChar cc;
         int ncc;
@@ -867,8 +872,8 @@ ScmObj Scm_StringSplitByChar(ScmString *str, ScmChar ch)
 }
 
 /* Boyer-Moore string search.  assuming siz1 > siz2, siz2 < 256. */
-static inline int boyer_moore(const char *ss1, int siz1,
-                              const char *ss2, int siz2)
+static int boyer_moore(const char *ss1, int siz1,
+                       const char *ss2, int siz2)
 {
     unsigned char shift[256];
     int i, j, k;
@@ -884,11 +889,128 @@ static inline int boyer_moore(const char *ss1, int siz1,
     return -1;
 }
 
-/* Scan s2 in s1.  If both strings are single-byte, and s1 is long,
-   we use Boyer-Moore.
-   
-   To avoid rescanning of the string, this function can return
-   various information, depends on retmode argument.
+static int boyer_moore_reverse(const char *ss1, int siz1,
+                               const char *ss2, int siz2)
+{
+    unsigned char shift[256];
+    int i, j, k;
+    for (i=0; i<256; i++) { shift[i] = siz2; }
+    for (j=siz2-1; j>0; j--) {
+        shift[(unsigned char)ss2[j]] = j;
+    }
+    for (i=siz1-siz2+1; i>=0; i-=shift[(unsigned char)ss1[i]]) {
+        for (j=0, k = i; j<siz2 && ss1[k] == ss2[j]; j++, k++)
+            ;
+        if (j == siz2) return i;
+    }
+    return -1;
+}
+
+/* A primitive routine to search a substring s2 in s1.
+   Returns TRUE iff found.  Calculates both byte index and character index.
+   If both strings are single-byte and s1 is long, we use Boyer-Moore.
+ */
+static int string_search(const char *s1, int siz1, int len1,
+                         const char *s2, int siz2, int len2,
+                         int *bi /* out */,
+                         int *ci /* out */)
+{
+    int i;
+
+    if (siz2 == 0) {
+        *bi = *ci = 0;
+        return TRUE;
+    }
+
+    if (siz1 == len1) {
+        if (siz2 == len2) {
+            /* short cut for single-byte strings */
+            if (siz1 < siz2) return FALSE;
+            if (siz1 < 256 || siz2 >= 256) {
+                /* brute-force search */
+                for (i=0; i<=siz1-siz2; i++) {
+                    if (memcmp(s2, s1+i, siz2) == 0) break;
+                }
+                if (i == siz1-siz2+1) return FALSE;
+            } else {
+                i = boyer_moore(s1, siz1, s2, siz2);
+                if (i < 0) return FALSE;
+            }
+            *bi = *ci = i;
+            return TRUE;
+        } else {
+            return FALSE;       /* sbstring can't contain mbstring. */
+        }
+    }
+    if (len1 >= len2) {
+        const char *sp = s1;
+        for (i=0; i<=len1-len2; i++) {
+            if (memcmp(sp, s2, siz2) == 0) {
+                *bi = (int)(sp - s1);
+                *ci = i;
+                return TRUE;
+            }
+            sp += SCM_CHAR_NFOLLOWS(*sp) + 1;
+        }
+    }
+    return FALSE;
+}
+
+static int string_search_reverse(const char *s1, int siz1, int len1,
+                                 const char *s2, int siz2, int len2,
+                                 int *bi /* out */,
+                                 int *ci /* out */)
+{
+    int i;
+
+    if (siz2 == 0) {
+        *bi = siz1;
+        *ci = len1;
+        return TRUE;
+    }
+
+    if (siz1 == len1) {
+        if (siz2 == len2) {
+            /* short cut for single-byte strings */
+            if (siz1 < siz2) return FALSE;
+            if (siz1 < 256 || siz2 >= 256) {
+                /* brute-force search */
+                for (i=siz1-siz2-1; i>=0; i--) {
+                    if (memcmp(s2, s1+i, siz2) == 0) break;
+                }
+                if (i == 0) return FALSE;
+            } else {
+                i = boyer_moore_reverse(s1, siz1, s2, siz2);
+                if (i < 0) return FALSE;
+            }
+            *bi = *ci = i;
+            return TRUE;
+        } else {
+            return FALSE;       /* sbstring can't contain mbstring. */
+        }
+    }
+    if (len1 >= len2) {
+        const char *sp = s1 + siz1, *p;
+        for (i=0; i<len2; i++) {
+            SCM_CHAR_BACKWARD(sp, s1, p);
+            SCM_ASSERT(*p);
+            sp = p;
+        }
+        for (i=len1-len2; i>=0; i--) {
+            if (memcmp(sp, s2, siz2) == 0) {
+                *bi = (int)(sp - s1);
+                *ci = i;
+                return TRUE;
+            }
+            SCM_CHAR_BACKWARD(sp, s1, p);
+            sp = p;
+        }
+    }
+    return FALSE;
+}
+
+/* Scan s2 in s1, and calculates appropriate return value(s) according to       
+   retmode.
 
    SCM_STRING_SCAN_INDEX  : return the index of s1
         s1 = "abcde" and s2 = "cd" => 2
@@ -903,13 +1025,15 @@ static inline int boyer_moore(const char *ss1, int siz1,
    SCM_STRING_SCAN_BOTH   : return substring of s1 before and after s2
        s1 = "abcde" and s2 = "cd" => "ab" and "e"
 */
-static ScmObj string_scan(ScmString *s1, const char *ss2,
+static ScmObj string_scan(ScmString *ss1, const char *s2,
                           int siz2, int len2, int incomplete2,
-                          int retmode)
+                          int retmode,
+                          int (*searcher)(const char*, int, int,
+                                          const char*, int, int, int*, int*))
 {
-    int i, incomplete;
-    const ScmStringBody *sb = SCM_STRING_BODY(s1);
-    const char *ss1 = SCM_STRING_BODY_START(sb);
+    int bi, ci, incomplete;
+    const ScmStringBody *sb = SCM_STRING_BODY(ss1);
+    const char *s1 = SCM_STRING_BODY_START(sb);
     int siz1 = SCM_STRING_BODY_SIZE(sb);
     int len1 = SCM_STRING_BODY_LENGTH(sb);
 
@@ -917,105 +1041,40 @@ static ScmObj string_scan(ScmString *s1, const char *ss2,
         Scm_Error("return mode out fo range: %d", retmode);
     }
 
-    if (siz2 == 0) {
-        /* shortcut */
-        switch (retmode) {
-        case SCM_STRING_SCAN_INDEX: return SCM_MAKE_INT(0);
-        case SCM_STRING_SCAN_BEFORE: return SCM_MAKE_STR("");
-        case SCM_STRING_SCAN_AFTER:  return Scm_CopyString(s1);
-        case SCM_STRING_SCAN_BEFORE2:;
-        case SCM_STRING_SCAN_AFTER2:;
-        case SCM_STRING_SCAN_BOTH:
-            return Scm_Values2(SCM_MAKE_STR(""), Scm_CopyString(s1));
+    if (!searcher(s1, siz1, len1, s2, siz2, len2, &bi, &ci)) {
+        if (retmode <= SCM_STRING_SCAN_AFTER) {
+            return SCM_FALSE;
+        } else {
+            return Scm_Values2(SCM_FALSE, SCM_FALSE);
         }
     }
-    
-    if (siz1 == len1) {
-        if (siz2 == len2) goto sbstring;
-        goto failed;            /* sbstring can't contain mbstring. */   
-    }
-    if (len1 >= len2) {
-        const char *ssp = ss1;
-        for (i=0; i<=len1-len2; i++) {
-            if (memcmp(ssp, ss2, siz2) == 0) {
-                switch (retmode) {
-                case SCM_STRING_SCAN_INDEX:
-                    return Scm_MakeInteger(i);
-                case SCM_STRING_SCAN_BEFORE:
-                    return Scm_MakeString(ss1, (int)(ssp-ss1), i, 0);
-                case SCM_STRING_SCAN_AFTER:
-                    return Scm_MakeString(ssp+siz2,
-                                          (int)(siz1-(ssp-ss1+siz2)),
-                                          len1-i-len2, 0);
-                case SCM_STRING_SCAN_BEFORE2:
-                    return Scm_Values2(Scm_MakeString(ss1, (int)(ssp-ss1),
-                                                      i, 0),
-                                       Scm_MakeString(ssp,
-                                                      (int)(siz1-(ssp-ss1)),
-                                                      len1-i, 0));
-                case SCM_STRING_SCAN_AFTER2:
-                    return Scm_Values2(Scm_MakeString(ss1,
-                                                      (int)(ssp-ss1+siz2),
-                                                      i+len2, 0),
-                                       Scm_MakeString(ssp+siz2,
-                                                      (int)(siz1-(ssp-ss1+siz2)),
-                                                      len1-i-len2, 0));
-                case SCM_STRING_SCAN_BOTH:
-                    return Scm_Values2(Scm_MakeString(ss1, (int)(ssp-ss1), i, 0),
-                                       Scm_MakeString(ssp+siz2,
-                                                      (int)(siz1-(ssp-ss1+siz2)),
-                                                      len1-i-len2, 0));
-                }
-            }
-            ssp += SCM_CHAR_NFOLLOWS(*ssp) + 1;
-        }
-    }
-    goto failed;
 
-  sbstring: /* short cut for single-byte strings */
-    if (siz1 < siz2) goto failed;
-    if (siz1 < 256 || siz2 >= 256) {
-        /* brute-force search */
-        for (i=0; i<=siz1-siz2; i++) {
-            if (memcmp(ss2, ss1+i, siz2) == 0) break;
-        }
-        if (i == siz1-siz2+1) goto failed;
-    } else {
-        i = boyer_moore(ss1, siz1, ss2, siz2);
-        if (i < 0) goto failed;
-    }
     incomplete =
         (SCM_STRING_BODY_INCOMPLETE_P(sb) || incomplete2)?
         SCM_STRING_INCOMPLETE : 0;
+    
     switch (retmode) {
     case SCM_STRING_SCAN_INDEX:
-        return Scm_MakeInteger(i);
+        return Scm_MakeInteger(ci);
     case SCM_STRING_SCAN_BEFORE:
-        return Scm_MakeString(ss1, i, i, incomplete);
+        return Scm_MakeString(s1, bi, ci, incomplete);
     case SCM_STRING_SCAN_AFTER:
-        return Scm_MakeString(ss1+i+siz2, siz1-(i+siz2), siz1-(i+siz2),
-                              incomplete);
+        return Scm_MakeString(s1+bi+siz2, siz1-bi-siz2,
+                              len1-ci-len2, incomplete);
     case SCM_STRING_SCAN_BEFORE2:
-        return Scm_Values2(Scm_MakeString(ss1, i, i, incomplete),
-                           Scm_MakeString(ss1+i, siz1-i, siz1-i, incomplete));
+        return Scm_Values2(Scm_MakeString(s1, bi, ci, incomplete),
+                           Scm_MakeString(s1+bi, siz1-bi, len1-ci, incomplete));
     case SCM_STRING_SCAN_AFTER2:
-        return Scm_Values2(Scm_MakeString(ss1, i+siz2, i+siz2, incomplete),
-                           Scm_MakeString(ss1+i+siz2, siz1-(i+siz2),
-                                          siz1-(i+siz2), incomplete));
+        return Scm_Values2(Scm_MakeString(s1, bi+siz2, ci+len2, incomplete),
+                           Scm_MakeString(s1+bi+siz2, siz1-bi-siz2,
+                                          len1-ci-len2, incomplete));
     case SCM_STRING_SCAN_BOTH:
-        return Scm_Values2(Scm_MakeString(ss1, i, i, incomplete),
-                           Scm_MakeString(ss1+i+siz2, siz1-(i+siz2),
-                                          siz1-(i+siz2), incomplete));
-    }
-  failed:
-    if (retmode <= SCM_STRING_SCAN_AFTER) {
-        return SCM_FALSE;
-    } else {
-        return Scm_Values2(SCM_FALSE, SCM_FALSE);
+        return Scm_Values2(Scm_MakeString(s1, bi, ci, incomplete),
+                           Scm_MakeString(s1+bi+siz2, siz1-bi-siz2,
+                                          len1-ci-len2, incomplete));
     }
 }
 
-
 ScmObj Scm_StringScan(ScmString *s1, ScmString *s2, int retmode)
 {
     const ScmStringBody *s2b = SCM_STRING_BODY(s2);
@@ -1024,14 +1083,34 @@ ScmObj Scm_StringScan(ScmString *s1, ScmString *s2, int retmode)
                        SCM_STRING_BODY_SIZE(s2b),
                        SCM_STRING_BODY_LENGTH(s2b),
                        SCM_STRING_BODY_INCOMPLETE_P(s2b),
-                       retmode);
+                       retmode, string_search);
 }
 
 ScmObj Scm_StringScanChar(ScmString *s1, ScmChar ch, int retmode)
 {
     char buf[SCM_CHAR_MAX_BYTES];
     SCM_CHAR_PUT(buf, ch);
-    return string_scan(s1, buf, SCM_CHAR_NBYTES(ch), 1, FALSE, retmode);
+    return string_scan(s1, buf, SCM_CHAR_NBYTES(ch), 1, FALSE, retmode,
+                       string_search);
+}
+
+ScmObj Scm_StringScanRight(ScmString *s1, ScmString *s2, int retmode)
+{
+    const ScmStringBody *s2b = SCM_STRING_BODY(s2);
+    return string_scan(s1,
+                       SCM_STRING_BODY_START(s2b),
+                       SCM_STRING_BODY_SIZE(s2b),
+                       SCM_STRING_BODY_LENGTH(s2b),
+                       SCM_STRING_BODY_INCOMPLETE_P(s2b),
+                       retmode, string_search_reverse);
+}
+
+ScmObj Scm_StringScanCharRight(ScmString *s1, ScmChar ch, int retmode)
+{
+    char buf[SCM_CHAR_MAX_BYTES];
+    SCM_CHAR_PUT(buf, ch);
+    return string_scan(s1, buf, SCM_CHAR_NBYTES(ch), 1, FALSE, retmode,
+                       string_search_reverse);
 }
 
 /*----------------------------------------------------------------
@@ -1218,7 +1297,7 @@ ScmObj Scm_MakeStringPointer(ScmString *src, int index, int start, int end)
     SCM_CHECK_START_END(start, end, len);
     while (index < 0) index += (end - start) + 1;
     if (index > (end - start)) goto badindex;
-    
+
     if (SCM_STRING_BODY_SINGLE_BYTE_P(srcb)) {
         sptr = SCM_STRING_BODY_START(srcb) + start;
         ptr = sptr + index;
@@ -1347,7 +1426,6 @@ ScmObj Scm_StringPointerCopy(ScmStringPointer *sp1)
 
 /* Dump string pointer info for debugging.
    Thanks to Alex Shinn (foof@synthcode.com) */
-#if SCM_DEBUG_HELPER
 void Scm_StringPointerDump(ScmStringPointer *sp1)
 {
     Scm_Printf(SCM_CUROUT,
@@ -1355,7 +1433,6 @@ void Scm_StringPointerDump(ScmStringPointer *sp1)
                sp1, sp1->length, sp1->size, sp1->start, sp1->index,
                sp1->current);
 }
-#endif /*SCM_DEBUG_HELPER*/
 
 /*==================================================================
  *
@@ -1432,9 +1509,9 @@ void Scm__DStringRealloc(ScmDString *dstr, int minincr)
     newchunk = SCM_NEW_ATOMIC2(ScmDStringChunk*,
                                sizeof(ScmDStringChunk)+newsize-SCM_DSTRING_INIT_CHUNK_SIZE);
     newchunk->bytes = 0;
-    
+
     newchain = SCM_NEW(ScmDStringChain);
-    
+
     newchain->next = NULL;
     newchain->chunk = newchunk;
     if (dstr->tail) {
@@ -1463,7 +1540,7 @@ static const char *dstring_getz(ScmDString *dstr, int *plen, int *psiz)
     } else {
         ScmDStringChain *chain = dstr->anchor;
         char *bptr;
-        
+
         size = Scm_DStringSize(dstr);
         len = dstr->length;
         bptr = buf = SCM_NEW_ATOMIC2(char*, size+1);
@@ -1541,7 +1618,6 @@ void Scm_DStringPutc(ScmDString *ds, ScmChar ch)
 
 
 /* for debug */
-#if SCM_DEBUG_HELPER
 void Scm_DStringDump(FILE *out, ScmDString *dstr)
 {
     fprintf(out, "DString %p\n", dstr);
@@ -1563,5 +1639,3 @@ void Scm_DStringDump(FILE *out, ScmDString *dstr)
         fprintf(out, "\"\n");
     }
 }
-#endif /*SCM_DEBUG_HELPER*/
-
